.nav li.dropdown ul.dropdown-menu li:hover ul {
	display:block;
	position:absolute;
	left:100%;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	border-radius: 3px;
}
.nav li.dropdown ul.dropdown-menu ul {
	display: none;
	float:right;
	position: relative;
	top: auto;
	margin-top: -30px;
}
.nav li.dropdown a.dropdown-toggle .glyphicon {
	margin: 0 4px;
}
