{% extends 'admin/master.html' %}
{% import 'admin/lib.html' as lib with context %}

{% block body %}
  {% block navlinks %}
  <ul class="nav nav-tabs">
    <li class="nav-item">
        <a class="nav-link" href="{{ return_url }}">{{ _gettext('List') }}</a>
    </li>
    {%- if admin_view.can_create -%}
    <li class="nav-item">
        <a class="nav-link" href="{{ get_url('.create_view', url=return_url) }}">{{ _gettext('Create') }}</a>
    </li>
    {%- endif -%}
    {%- if admin_view.can_edit -%}
    <li class="nav-item">
        <a class="nav-link" href="{{ get_url('.edit_view', id=request.args.get('id'), url=return_url) }}">{{ _gettext('Edit') }}</a>
    </li>
    {%- endif -%}
    <li class="nav-item">
        <a class="nav-link active disabled" href="javascript:void(0)">{{ _gettext('Details') }}</a>
    </li>
  </ul>
  {% endblock %}

  {% block details_search %}
    <div class="form-inline fa_filter_container col-lg-6">
      <label for="fa_filter">{{ _gettext('Filter') }}</label>
      <input id="fa_filter" type="text" class="ml-3 form-control">
    </div>
  {% endblock %}

  {% block details_table %}
    <table class="table table-hover table-bordered searchable">
    {% for c, name in details_columns %}
      <tr>
        <td>
          <b>{{ name }}</b>
        </td>
        <td>
        {{ get_value(model, c) }}
        </td>
      </tr>
    {% endfor %}
    </table>
  {% endblock %}
{% endblock %}

{% block tail %}
  {{ super() }}
  <script src="{{ admin_static.url(filename='admin/js/details_filter.js', v='1.0.0') }}"></script>
{% endblock %}
