{% extends 'admin/model/create.html' %}
{% import 'another_macro.html' as test_lib %}

{% macro test(arg) %}
  Value = {{ arg }}
{% endmacro %}

{% macro wrap() %}
  <wrapper>
    {{ caller() }}
  </wrapper>
{% endmacro %}
